/*
	This file is part of cpso.

	cpso is free software: you can redistribute it and/or modify
	it under the terms of the GNU Lesser General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.

	cpso is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU Lesser General Public License for more details.

	You should have received a copy of the GNU Lesser General Public License
	along with cpso.  If not, see <http://www.gnu.org/licenses/>.
    
    Copyright (C) 2010 Lucas Hermann Negri
*/

#include <cpso.h>
#include <stdio.h>
#include <assert.h>
#include <math.h>
#include <time.h>

double eval(CpsoState* st, const double* p)
{
    double x = p[0], y = p[1];
    
    /* Rastrigin's function */
    double val = 20 + (x * x - 10 * cos(2 * M_PI * x)) + (y * y - 10 * cos(2 * M_PI * y));
    
    return -val; /* minimization */
}

int main(int argc, char* argv[])
{
    CpsoState* st = cpso_new(2, 10);
    
    cpso_set_limits(st, 0, -10, 10);
    cpso_set_limits(st, 1, -10, 10);
    cpso_set_eval  (st, &eval, NULL);
    cpso_set_params(st, 1.8, 1.5, 0.55, 0.015);
    srand( time(0) );
    cpso_init(st);
    
    int i = cpso_run(st, 10000, -1e-6);
    
    const double* best;
    double fit = cpso_get_best(st, -1, &best);
    printf("** Best particle **\n"                   );
    printf("Iterations = %d\n"     , i               );
    printf("Fitness    = %f\n"     , fit             );
    printf("x, y       = %f, %f\n" , best[0], best[1]);
    
    cpso_free(st);
    
    return 0;
}
